@@ -81,12 +81,16 @@ CREATE OR REPLACE PROCEDURE @extschema@.remove_columnstore_policy(
 
 /* continuous aggregates policy */
 CREATE OR REPLACE FUNCTION @extschema@.add_continuous_aggregate_policy(
-    continuous_aggregate REGCLASS, start_offset "any",
-    end_offset "any", schedule_interval INTERVAL,
+    continuous_aggregate REGCLASS,
+    start_offset "any",
+    end_offset "any",
+    schedule_interval INTERVAL,
     if_not_exists BOOL = false,
     initial_start TIMESTAMPTZ = NULL,
     timezone TEXT = NULL,
-    include_tiered_data BOOL = NULL
+    include_tiered_data BOOL = NULL,
+    buckets_per_batch INTEGER = NULL,
+    max_batches_per_execution INTEGER = NULL
 )
 RETURNS INTEGER
 AS '@MODULE_PATHNAME@', 'ts_policy_refresh_cagg_add'
