@@ -190,6 +190,9 @@ extern void ts_chunk_recreate_all_constraints_for_dimension(Hypertable *ht, int3
 extern int ts_chunk_delete_by_hypertable_id(int32 hypertable_id);
 extern TSDLLEXPORT int ts_chunk_delete_by_name(const char *schema, const char *table,
 											   DropBehavior behavior);
+extern int ts_chunk_delete_by_relid_and_relname(Oid relid, const char *schemaname,
+												const char *tablename, DropBehavior behavior,
+												bool preserve_chunk_catalog_row);
 extern bool ts_chunk_set_name(Chunk *chunk, const char *newname);
 extern bool ts_chunk_set_schema(Chunk *chunk, const char *newschema);
 extern TSDLLEXPORT List *ts_chunk_get_window(int32 dimension_id, int64 point, int count,
