@@ -91,6 +91,8 @@ cleanup_on_pg_proc_exit(int code, Datum arg)
 void
 _PG_init(void)
 {
+	static bool init_done = false;
+
 	/*
 	 * Check extension_is loaded to catch certain errors such as calls to
 	 * functions defined on the wrong extension version
@@ -99,6 +101,12 @@ _PG_init(void)
 	ts_extension_check_server_version();
 	ts_bgw_check_loader_api_version();
 
+	/* We can call _PG_init() several times if we do an eager load, so abort
+	 * init if we do. */
+
+	if (init_done)
+		return;
+
 	_cache_init();
 	_hypertable_cache_init();
 	_cache_invalidate_init();
@@ -118,6 +126,7 @@ _PG_init(void)
 
 	/* Register a cleanup function to be called when the backend exits */
 	on_proc_exit(cleanup_on_pg_proc_exit, 0);
+	init_done = true;
 }
 
 TSDLLEXPORT Datum
