@@ -33,7 +33,13 @@ ts_compression_chunk_size_delete(int32 uncompressed_chunk_id)
 	ts_scanner_foreach(&iterator)
 	{
 		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
-		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+		ts_catalog_delete_tid_only(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+		count++;
 	}
+
+	/* Make catalog changes visible */
+	if (count > 0)
+		CommandCounterIncrement();
+
 	return count;
 }
